@@ -102,6 +102,17 @@
 #' @param sigma Numeric value to use for subsequent predictions or
 #'   back-transformation bias adjustments. If not specified, we use
 #'   \code{sigma(object)}, if available, and \code{NULL} otherwise.
+#' @param nuisance,non.nuisance,wt.nuis If \code{nuisance} is a vector of predictor names,
+#'   those predictors are omitted from the reference grid. Instead, the result 
+#'   will be as if we had averaged over the levels of those factors, with either 
+#'   equal or proportional weights as specified in \code{wt.nuis} (see the 
+#'   \code{weights} argument in \code{\link{emmeans}}). The factors in 
+#'   \code{nuisance} must not interact with other factors, not even other
+#'   nuisance factors. Specifying nuisance factors can save considerable
+#'   storage and computation time, and help avoid exceeding the maximum
+#'   reference-grid size (\code{get_emm_option("rg.limit")}).
+#' @param rg.limit Integer limit on the number of reference-grid rows to allow
+#'   (checked before any multivariate responses are included).
 #' @param ... Optional arguments passed to \code{\link{summary.emmGrid}},
 #'   \code{\link{emm_basis}}, and
 #'   \code{\link{recover_data}}, such as \code{params}, \code{vcov.} (see
@@ -370,7 +381,9 @@
 ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("cov.keep"),
                      mult.names, mult.levs, 
                      options = get_emm_option("ref_grid"), data, df, type, 
-                     transform, nesting, offset, sigma, ...) 
+                     transform, nesting, offset, sigma, 
+                     nuisance = character(0), non.nuisance, wt.nuis = "equal", 
+                     rg.limit = get_emm_option("rg.limit"), ...) 
 {
     ### transform = match.arg(transform)
     if (!missing(df)) {
@@ -518,8 +531,18 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
     }
     
 
+    if (!missing(non.nuisance))
+        nuisance = setdiff(names(ref.levels), non.nuisance)
+
     # Now create the reference grid
-    grid = do.call(expand.grid, ref.levels)
+    if(no.nuis <- (length(nuisance) == 0)) {
+        .check.grid(ref.levels, rg.limit)
+        grid = do.call(expand.grid, ref.levels)
+    }
+    else {
+        nuis.info = .setup.nuis(nuisance, ref.levels, trms, rg.limit)
+        grid = nuis.info$grid
+    }
     
     # undocumented hook to expand grid by increments of 'var' (needed by emtrends)
     if (!is.null(delts <- options$delts)) {
@@ -564,14 +587,32 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
     if (!is.null(attr(data, "pass.it.on")))   # a hook needed by emm_basis.gamlss et al.
         attr(object, "data") = data
     
+    ###!! Prevent a warning like in https://stackoverflow.com/questions/68969384/emmeans-warning-in-model-frame-defaultformula-data-data-variable-gr/68990172#68990172
+    xl = xlev
+    modnm = rownames(attr(trms, "factors"))
+    chk = sapply(modnm, function(mn) mn %in% names(xl))
+    for(i in which(!chk)) { # replace names in xl - e.g., as.factor(trt) where trt already a factor
+        fn = all.vars(reformulate(modnm[i]))
+        if (length(fn) == 1)
+            names(xl)[names(xl) == fn] = modnm[i]
+    }
+    ###!! If we remove this code, also need to change xl back to xlev in 'basis =' call below
+    
     # we've added args `misc` and `options` so emm_basis methods can access and use these if they want
-    basis = emm_basis(object, trms, xlev, grid, misc = attr(data, "misc"), options = options, ...)
+    basis = emm_basis(object, trms, xl, grid, misc = attr(data, "misc"), options = options, ...)
     environment(basis$dffun) = baseenv()   # releases unnecessary storage
     if(length(basis$bhat) != ncol(basis$X))
         stop("Something went wrong:\n",
              " Non-conformable elements in reference grid.",
              call. = TRUE)
     
+    if(!no.nuis) {
+        basis = .basis.nuis(basis, nuis.info, wt.nuis, ref.levels, data, grid, ref.levels)
+        grid = basis$grid
+        nuisance = ref.levels[nuis.info$nuis] # now nuisance has the levels info
+        ref.levels = basis$ref.levels
+    }
+    
     misc = basis$misc
     
     ### Figure out if there is a response transformation...
@@ -713,6 +754,7 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
         data[["(weights)"]] = 1
     cov.keep = intersect(unique(cov.keep), names(ref.levels))
     nms = union(union(union(names(xlev), names(chrlev)), coerced$factors), cov.keep)
+    nms = intersect(nms, names(grid))
     #### Old code...
     # if (!covnest)
     #     nms = union(union(names(xlev), names(chrlev)), coerced$factors) # only factors, no covariates or mult.resp
@@ -721,7 +763,7 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
     if (length(nms) == 0)
         wgt = rep(1, nrow(grid))  # all covariates; give each weight 1
     else {
-        id = plyr::id(data[, nms, drop = FALSE], drop = TRUE)
+        id = .my.id(data[, nms, drop = FALSE])
         uid = !duplicated(id)
         key = do.call(paste, unname(data[uid, nms, drop = FALSE]))
         key = key[order(id[uid])]
@@ -750,14 +792,15 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
     misc$level = .95
     misc$adjust = "none"
     misc$famSize = nrow(grid)
-    misc$avgd.over = character(0)
+    if(is.null(misc$avgd.over))
+       misc$avgd.over = character(0)
     misc$sigma = sigma
 
     post.beta = basis$post.beta
     if (is.null(post.beta))
         post.beta = matrix(NA)
     
-    predictors = attr(data, "predictors")
+    predictors = intersect(attr(data, "predictors"), names(grid))
     
     simp.tbl = environment(trms)$.simplify.names.
     if (! is.null(simp.tbl)) {
@@ -777,7 +820,8 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
          model.info = model.info,
          roles = list(predictors = predictors, 
                       responses = attr(data, "responses"), 
-                      multresp = multresp),
+                      multresp = multresp,
+                      nuisance = nuisance),
          grid = grid, levels = ref.levels, matlevs = matlevs,
          linfct = basis$X, bhat = basis$bhat, nbasis = basis$nbasis, V = basis$V,
          dffun = basis$dffun, dfargs = basis$dfargs, 
@@ -873,5 +917,112 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
     list(factors = cfac, covariates = ccov, orig = orig)
 }
 
+# My replacement for plyr::id(, drop = TRUE)
+.my.id = function(data){
+    p = do.call(paste, data)
+    u = unique(p)
+    match(p, u)
+}
 
+# Utility to error-out when potential reference grid size is too big
+.check.grid = function(levs, limit = get_emm_option("rg.limit")) {
+    size = prod(sapply(levs, length))
+    if (size > limit)
+        stop("The rows of your requested reference grid would be ", size, ", which exceeds\n",
+             "the limit of ", limit, " (not including any multivariate responses).\n",
+             "Your options are:\n",
+             "  1. Specify some (or more) nuisance factors using the 'nuisance' argument\n",
+             "     (see ?ref_grid). These must be factors that do not interact with others.\n",
+             "  2. Add the argument 'rg.limit = <new limit>' to the call. Be careful,\n",
+             "     because this could cause excessive memory use and performance issues.\n",
+             "     Or, change the default via 'emm_options(rg.limit = <new limit>)'.\n",
+             call. = FALSE)
+}
+
+# Utility to set up the grid for nuisance factors. This consists of two or more
+# data.frames rbinded together:
+#   * the expanded grid for all factors *not* in nuis, with the nuis factors set 
+#     at their first levels
+#   * for each factor in f nuis, a set of rows for (levs$f), with the other 
+#     factors at their first levels (this is arbitrary)
+# In addition, we return a character vector 'row.assign' corresponding to the rows 
+# in the grid, telling which is what: ".main.grid." for the first part of the grid,
+# otherwise factor names from nuis.
+# We also return 'nuis' itself - which may be reduced since illegal 
+# entries are silently removed
+.setup.nuis = function(nuis, levs, trms, rg.limit) {
+    firsts =  args = lapply(levs, function(x) x[1])
+    nuis = intersect(nuis, names(levs))
+    # sanity checks on terms, and term indexes
+    fsum = rep(99, length(nuis))
+    tbl = attr(trms, "factors")
+    rn = row.names(tbl) = sapply(row.names(tbl), function(nm)
+        paste(all.vars(reformulate(nm)), collapse = ","))
+    for (i in seq_along(nuis)) {
+        f = nuis[i]
+        if(f %in% rn)
+            fsum[i] = sum(tbl[f, ])
+    }
+    nuis = nuis[fsum == 1]   # silently remove any unfound or interacting factors
+
+    # top part...
+    non.nuis = setdiff(names(levs), nuis)
+    for (n in non.nuis)
+        args[[n]] = levs[[n]]
+    .check.grid(args, rg.limit)
+    grid = do.call(expand.grid, args)
+    ra = rep(".main.grid.", nrow(grid))
+    # bottom parts
+    for (f in nuis) {
+        args = firsts
+        args[[f]] = levs[[f]]
+        grid = rbind(grid, do.call(expand.grid, args))
+        ra = c(ra, rep(f, length(levs[[f]])))
+    }
+    list(grid = grid, row.assign = ra, nuis = nuis)
+}
+
+# Do the required post-processing for nuisance factors...
+# After we get the model matrix for this grid, we'll average each set of rows in the
+# bottom part, and substitute those averages in the required columns in the top part 
+# of the model matrix.
+.basis.nuis = function(basis, info, wt, levs, data, grid, ref.levels) {
+    ra = info$row.assign
+    r. = rep(".", length(ra))  # fillers
+    X = basis$X
+    n = sum(ra == ".main.grid.")
+    k = nrow(X) / length(ra)   # multivariate dimension
+    nuis = info$nuis
+    wts = lapply(nuis, function(f) {
+        if (wt == "equal")
+            w = rep(1, length(levs[[f]]))
+        else {
+            x = data[[f]]
+            w = sapply(levs[[f]], function(lev) sum(x == lev))
+        }
+        w / sum(w)
+    })
+    names(wts) = nuis
+    
+    # In a multivariate case, we have to repeat the same operations for each block of X rows
+    for (m in 1:k) {
+        RA = c(rep(r., m - 1), ra, rep(r., k - m))
+        for (f in nuis) {
+            subX = X[RA == f, , drop = FALSE]
+            cols = which(apply(subX, 2, function(x) diff(range(x)) > 0))
+            subX = sweep(subX[, cols, drop = FALSE], 1, wts[[f]], "*")
+            avg = apply(subX, 2, sum)
+            avg = matrix(rep(avg, each = n), nrow = n) # now several copies
+            X[RA == ".main.grid.", cols] = avg
+        }
+    }
+    basis$misc$nuis = nuis
+    basis$misc$avgd.over = paste(length(nuis), "nuisance factors")
+    RA = rep(ra, k)
+    basis$X = X[RA == ".main.grid.", , drop = FALSE]
+    non.nuis = setdiff(names(ref.levels), info$nuis)
+    basis$ref.levels = ref.levels[non.nuis]
+    basis$grid = grid[1:n, non.nuis, drop = FALSE]
+    basis
+}
 
